# -*- coding: utf-8 -*-

from numpy import zeros, array, arange, meshgrid, abs as np_abs, max as np_max

from ...Functions.Plot.plot_4D import plot_4D


def plot_modal_cont(
    modal_cont,
    mode_labels,
    nat_freqs,
    ifrq,
    freq=None,
    save_path=None,
    is_show_fig=True,
    label="",
):
    """Plot the result of the modal contribution at a given frequency

    Parameters
    ----------
    modal_cont : dict
        result of the modal contribution
    mode_labels : list
        list of mode labels
    nat_freqs : list
        list of natural frequencies
    ifrq : int
        index of freq to use for the plot
    freq : float
        requested frequency
    save_path : str
        location to store the figure
    is_show_fig : bool
        True to call show at the end of the plot
    label : str
        name of the FRF
    """
    modal_cont_ext = modal_cont["expansion_ext"]
    nmodes = len(mode_labels)
    nloads = len(list(modal_cont_ext.keys()))
    zmax = 0
    Zdatas = zeros((nloads, nmodes))
    annot = zeros((nloads, nmodes), dtype=int)

    if freq is not None:
        # single frequency
        title = "Modal contribution of " + label + " at " + str(freq) + " Hz"
    else:
        title = "Modal contribution of " + label

    i_lc = 0
    yticklabels = list()
    for lc in modal_cont_ext.keys():
        modal_cont_lc = array(modal_cont_ext[lc])
        modal_cont_ind = [int(x) for x in array(modal_cont_ext[lc])[:, 0].tolist()]
        Zdatas[i_lc, modal_cont_ind] = np_abs(modal_cont_lc[:, ifrq + 1])
        yticklabels.append(lc)
        annot[i_lc, modal_cont_ind] = (array(nat_freqs)[modal_cont_ind]).astype(int)
        i_lc += 1

    x = arange(nmodes)
    y = arange(nloads)
    x_map, y_map = meshgrid(x, y)
    x_flat = x_map.flatten()
    y_flat = y_map.flatten()
    zmax = np_max(Zdatas)
    annotations = annot.flatten()

    size = 1000 * Zdatas / zmax
    Zdata_flat = Zdatas.flatten()
    size_flat = size.flatten()

    xticklabels = mode_labels

    plot_4D(
        x_flat,
        y_flat,
        Zdata_flat,
        size_flat,
        z_max=zmax,
        z_min=0,
        xticks=x,
        yticks=y,
        xticklabels=xticklabels,
        yticklabels=yticklabels,
        xlabel="Mode",
        ylabel="Loadcase",
        zlabel=label,
        annotations=annotations,
        title=title,
        type="scatter",
        save_path=save_path,
        is_show_fig=is_show_fig,
    )



# -*- coding: utf-8 -*-

from itertools import repeat

import matplotlib.pyplot as plt
from numpy import (
    argmin,
    abs,
    squeeze,
    split,
    ndarray,
    cumsum,
    zeros,
    shape,
    concatenate,
    linspace,
    pi,
    cos,
    sin,
    real,
    exp,
    ones_like,
)
from numpy import gcd

from glob import glob
from os import remove
from os.path import join

import imageio

# from pyleecan.definitions import config_dict

# # Import values from config dict
# FONT_NAME = config_dict["PLOT"]["FONT_NAME"]
# FONT_SIZE_TITLE = config_dict["PLOT"]["FONT_SIZE_TITLE"]
# FONT_SIZE_LABEL = config_dict["PLOT"]["FONT_SIZE_LABEL"]
# FONT_SIZE_LEGEND = config_dict["PLOT"]["FONT_SIZE_LEGEND"]
# COLORS = config_dict["PLOT"]["COLOR_DICT"]["CURVE_COLORS"]


def plot_waves(
    wavenumbers,
    frequencies,
    Zs=48,
    p=4,
    fs=50,
    is_LF=True,
    legend_list=None,
    color_list=None,
    linestyle_list=["-"],
    linewidth_list=[2],
    title="",
    is_disp_title=True,
    win_title=None,
    save_path="./",
    file_name="waves.gif",
    nframe=25,
):
    """Plots a wave graph

    Parameters
    ----------
    wavenumbers : list
        list of wavenumbers
    frequencies : list
        list of frequencies
    Zs : int
        number of slots
    p : int
        number of pole pairs
    is_LF : bool
        the last quantity is lumped forces -> stator + arrows
    legend_list : list
        list of legends
    color_list : list
        list of colors to use for each curve
    linestyle_list : list
        list of line styles to use for each curve
    linewidth_list : list
        list of line width to use for each curve
    title : str
        title of the graph
    is_disp_title : bool
        boolean indicating if the title must be displayed
    win_title : str
        Title of the plot window
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    file_name : str
        name of the gif file
    nframe : int
        number of frames
    """

    # Number of curves on a axe
    ndatas = len(wavenumbers)

    # Expend default argument
    if 1 == len(color_list) < ndatas:
        # Set the same color for all curves
        color_list = list(repeat(color_list[0], ndatas))
    if 1 == len(linewidth_list) < ndatas:
        # Set the same color for all curves
        linewidth_list = list(repeat(linewidth_list[0], ndatas))
    if 1 == len(linestyle_list) < ndatas:
        # Set the same linestyles for all curves
        linestyle_list = list(repeat(linestyle_list[0], ndatas))
    if legend_list is None:
        legend_list = [
            "f = " + str(f) + " [Hz], r = " + str(wavenumbers[i])
            for i, f in enumerate(frequencies)
        ]

    # Plot
    theta = linspace(0, 2 * pi, 42 * Zs, endpoint=False)
    r = 10 + 4 * (ndatas - 1)
    stator_int = [
        r * 1.1 if ((i % 42) > 5 and (i % 42) < 37) else r for i in range(42 * Zs)
    ]
    stator_ext = [r * 1.2 for i in range(42 * Zs)]
    # rotor = [r-0.5 if ((i%252)>10 and (i%252)<242) else r-1 for i in range(42*Zs)]
    theta2 = linspace(0, 2 * pi, Zs, endpoint=False)
    r2 = [r for i in range(Zs)]
    with imageio.get_writer(join(save_path, file_name), mode="I") as writer:
        for t in linspace(0.0, 1.0 / fs, nframe + 1)[:nframe]:
            fig, ax = plt.subplots(figsize=(6, 6), subplot_kw=dict(polar=True))
            for i, r in enumerate(wavenumbers):
                radius = 10 + 4 * i
                if r == 0:
                    amp = 1
                else:
                    amp = 1
                if r == 0 and frequencies[i] == 0:
                    wave = [radius for j in range(42 * Zs)]
                else:
                    wave = radius + amp * cos(2 * pi * frequencies[i] * t + r * theta)
                if i < (ndatas - 1):
                    ax.plot(
                        theta, real(wave), color=color_list[i], label=legend_list[i]
                    )
                elif is_LF:
                    ax.plot(
                        theta,
                        real(wave),
                        color=color_list[i],
                        linestyle="--",
                        label=legend_list[i],
                    )
                else:
                    ax.plot(
                        theta, real(wave), color=color_list[i], label=legend_list[i]
                    )
            if is_LF:
                ax.fill_between(theta, stator_int, stator_ext, color="#E1E1E0")
                # ax.fill_between(theta, 0, rotor, color="#E8E9E8")
                wave2 = 1 * cos(2 * pi * frequencies[-1] * t + wavenumbers[-1] * theta2)
                ax.quiver(
                    theta2,
                    r2,
                    real(wave2) * cos(theta2),
                    real(wave2) * sin(theta2),
                    color=color_list[i],
                    scale_units="x",
                    scale=6,
                    headlength=2,
                    headaxislength=2,
                )

            bbox_props = dict(boxstyle="round,pad=0.3", fc="white", ec="k", lw=0.5)
            ax.text(
                0, 12, r"$\, x$", ha="center", va="center", size=10, bbox=bbox_props
            )
            ax.text(0, 16, r"$\!=$", ha="center", va="center", size=10, bbox=bbox_props)
            ax.text(0, 20, r"$+$", ha="center", va="center", size=10, bbox=bbox_props)
            ax.text(
                0,
                24,
                r"$\Rightarrow$",
                ha="center",
                va="center",
                size=10,
                bbox=bbox_props,
            )

            if is_disp_title:
                ax.set_title(title)

            plt.tight_layout()
            ax.legend(loc="center", prop={"family": FONT_NAME, "size": 6})
            if is_LF:
                ax.set_rticks([10 + 4 * i for i in range(ndatas - 1)])
            else:
                ax.set_rticks([10 + 4 * i for i in range(ndatas)])
            ax.set_xticks([0, pi / 2, pi, 3 * pi / 2])
            # ax.set_xticks([])
            ax.set_xticklabels([])
            ax.set_yticklabels([])
            ax.set_ylim([0, 11 + 4 * ndatas])
            ax.spines["polar"].set_visible(False)
            save_path_temp = save_path + "/temp_" + str(t) + ".png"
            fig.savefig(save_path_temp)
            plt.close()
            image = imageio.imread(save_path_temp)
            writer.append_data(image)

    for file in glob(save_path + "/temp_*"):
        remove(file)

    # ax.title.set_fontname(FONT_NAME)
    # ax.title.set_fontsize(FONT_SIZE_TITLE)

    # if win_title:
    #     fig.canvas.set_window_title(win_title)


def plot_waves_cart(
    wavenumbers,
    frequencies,
    Zs=48,
    Zr=None,
    p=4,
    rot_dir=-1,
    fs=50,
    is_LF=True,
    legend_list=None,
    color_list=None,
    linestyle_list=["-"],
    linewidth_list=[2],
    title="",
    is_disp_title=True,
    win_title=None,
    save_path="./",
    file_name="waves.gif",
):
    """Plots a wave graph

    Parameters
    ----------
    wavenumbers : list
        list of wavenumbers
    frequencies : list
        list of frequencies
    Zs : int
        number of slots
    p : int
        number of pole pairs
    is_LF : bool
        the last quantity is lumped forces -> stator + arrows
    legend_list : list
        list of legends
    color_list : list
        list of colors to use for each curve
    linestyle_list : list
        list of line styles to use for each curve
    linewidth_list : list
        list of line width to use for each curve
    title : str
        title of the graph
    is_disp_title : bool
        boolean indicating if the title must be displayed
    win_title : str
        Title of the plot window
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    file_name : str
        name of the gif file
    nframe : int
        number of frames
    """

    # Number of curves on a axe
    ndatas = len(wavenumbers)
    if is_LF:
        ndatas = ndatas - 1

    # Plot
    # Set discretization
    if Zr is not None:
        npoints = Zs * Zr * 4
        per_min = gcd.reduce([9 * w for w in wavenumbers if w != 0] + [Zs, Zr])
        nframe = int(npoints / (2 * Zr))
    else:
        npoints = Zs * p * 8
        per_min = gcd.reduce([p * w for w in wavenumbers if w != 0] + [Zs, p])
        nframe = int(npoints / (4 * p))

    theta = linspace(0, 2 * pi, npoints, endpoint=False)

    # Stator
    stator_int = [1.5 * ndatas + 2.5 for i in range(npoints)]
    stator_ext = [1.5 * ndatas + 3.5 for i in range(npoints)]
    per = npoints / (2 * Zs)
    teeth = [
        1.5 * ndatas + 2 if (i + per / 2) % (2 * per) >= per else 1.5 * ndatas + 2.5
        for i in range(npoints)
    ]
    # Rotor
    rotor_int = [0 for i in range(npoints)]
    rotor_ext = [1 for i in range(npoints)]

    time = linspace(0.0, 1.0 / fs, nframe + 1, endpoint=True)

    with imageio.get_writer(join(save_path, file_name), mode="I") as writer:
        for j in range(nframe):
            t = time[j]
            fig, ax = plt.subplots(figsize=(12, 6))
            if Zr is not None:
                per = npoints / (2 * Zr)
                magnets = [
                    1.5 if (i + nframe / 2 - 2 * j) % (2 * nframe) >= nframe else 1
                    for i in range(npoints)
                ]
            else:
                per = npoints / (4 * p)
                magnets = [
                    1.5 if (i + nframe / 2 + 2 * j) % (2 * nframe) >= nframe else 1
                    for i in range(npoints)
                ]

            ax.fill_between(theta, rotor_int, rotor_ext, color="#bbcf1c")
            if Zr is not None:
                ax.fill_between(theta, magnets, rotor_ext, color="#bbcf1c")
            else:
                ax.fill_between(theta, magnets, rotor_ext, color="#dcdcda")
            ax.fill_between(theta, stator_ext, stator_int, color="#0069a1")
            ax.fill_between(theta, stator_int, teeth, color="#0069a1")

            # Waves
            for i in range(ndatas):
                wave = (
                    1.5 * (i + 1)
                    + 1
                    + 0.5
                    * cos(2 * pi * frequencies[i] * fs * t + wavenumbers[i] * p * theta)
                )
                ax.plot(
                    theta,
                    wave,
                    "-o",
                    markevery=[int(npoints / per_min / 2)],
                    color=color_list[i],
                )
            # Lumped forces
            if is_LF:
                wave = (
                    1.5 * ndatas
                    + 2
                    + 0.5
                    * cos(
                        2 * pi * frequencies[i] * fs * t + wavenumbers[-1] * p * theta
                    )
                )
                ax.plot(
                    theta,
                    wave,
                    marker="o",
                    markevery=[int(npoints / per_min / 2)],
                    color=color_list[-1],
                    linestyle="--",
                )
                theta2 = linspace(pi / Zs, 2 * pi - pi / Zs, Zs, endpoint=True)
                wave2 = 0.5 * cos(
                    2 * pi * frequencies[i] * fs * t + wavenumbers[-1] * p * theta2
                )
                ax.quiver(
                    theta2,
                    [1.5 * ndatas + 2 for j in range(Zs)],
                    [0 for j in range(Zs)],
                    wave2,
                    color=color_list[-1],
                    scale_units="y",
                    scale=1,
                    minshaft=2,
                )

            # Reduce to minimum period
            ax.set_xlim([-theta[int(npoints / per_min)], theta[int(npoints / per_min)]])

            ax.axis("off")

            # Add legend
            for i, leg in enumerate(legend_list):
                if is_LF and i == ndatas:
                    label = leg
                else:
                    if frequencies[i] == 0:
                        f = "0"
                    elif frequencies[i] == 1:
                        f = "f_s"
                    else:
                        f = str(frequencies[i]) + "f_s"
                    if wavenumbers[i] == 0:
                        r = "0"
                    elif wavenumbers[i] == 1:
                        r = "p"
                    else:
                        r = str(wavenumbers[i]) + "p"
                    label = leg + r": $f=" + f + "$, $r=" + r + "$"
                ax.annotate(
                    label,
                    (0 - theta[int(npoints / per_min)] / 20, 1.5 * (i + 1) + 0.8),
                    ha="right",
                    color=color_list[i],
                )

            save_path_temp = save_path + "/temp_" + format(j, ".4g") + ".png"
            fig.savefig(save_path_temp)
            plt.close()
            image = imageio.imread(save_path_temp)
            writer.append_data(image)

    for file in glob(save_path + "/temp_*"):
        remove(file)


plot_waves_cart(
    [5, -5, -10, 0, 0],
    [5, 1, 4, 6, 6],
    Zs=48,
    Zr=None,
    p=4,
    color_list=["#bbcf1c", "#0069a1", "#da3061", "#da3061", "k"],
    is_LF=True,
    legend_list=[
        "Rotor flux",
        "Stator flux",
        "Maxwell stress #1",
        "Maxwell stress #2",
        "Lumped forces #2",
    ],
)



from numpy import transpose, concatenate, nan, max as np_max

from ...Functions.Plot.plot_2D import plot_2D
from ...definitions import config_dict

# Get default colors, fonts and linestyles
COLOR_DICT = config_dict["PLOT"]["COLOR_DICT"]["CURVE_COLORS"]
FONT_NAME = config_dict["PLOT"]["FONT_NAME"]
LINE_STYLE = config_dict["PLOT"]["LINE_STYLE"]


def plot_order_analysis(
    speeds,
    A_overall,
    A_orders,
    legend_list,
    db_min=10,
    db_max=100,
    save_path=None,
    xlabel="Rotation speed [rpm]",
    ylabel="Acoustic power [dBA]",
    title="Order tracking analysis",
    is_disp_title=True,
    is_grid=True,
    is_show_fig=True,
    fig=None,
    ax=None,
    is_db=True,
):
    """
    Plot spectrogram from a variable speed simulation

    speeds: ndarray
        Array of speed values
    freqs : ndarray
        Array of frequency vectors for each speed
    A_overall : ndarray
        maximum level function of speed
    A_orders : ndarray
        Amplitude array per speed and order
    legend_list : list
        Legend for each order
    db_min : float
        lines under db_min are not displayed
    db_max : float
        maximum dB scale
    save_path : str
        location to store the figure
    xlabel : str
        label for the x-axis
    ylabel : str
        label for the y-axis
    title : str
        title of the graph
    is_disp_title : bool
        boolean indicating if the title must be displayed
    is_grid : bool
        boolean indicating if the grid must be displayed
    is_show_fig : bool
        True to call show at the end of the plot
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        ax on which to plot the data

    Returns
    -------
        ax : axes

    Raises
    ------
    PlotError
    """

    # Number of orders
    Norders = A_orders.shape[1]

    # Build the color and linestyle lists
    # Copy default COLOR_DICT
    color_list = COLOR_DICT.copy()
    # Duplicate first line style Ncolor times
    ii = 0
    linestyle_list = [LINE_STYLE[ii] for color in COLOR_DICT]

    # Append color and linestyle lists while there are less colors than curves to plot
    while len(color_list) < Norders:
        color_list.extend(COLOR_DICT.copy())
        ii = ii + 1
        linestyle_list.extend([LINE_STYLE[ii] for color in COLOR_DICT])

    # Concatenate orders amplitude and overall value
    Ydatas = concatenate((A_overall[:, None], A_orders), axis=1)

    # Append black color to the color and line_style list
    colors = ["#000000"] + color_list
    linestyles = [LINE_STYLE[0]] + linestyle_list
    legends = ["Overall"] + legend_list

    # Put zeros to NaN so they are not plotted
    Ydatas[Ydatas == 0] = nan

    # Set maximum value on y-axis
    if is_db:
        y_min = db_min
        y_max = 1.05 * np_max(A_overall)
    else:
        y_min = None
        y_max = None

    # Plot 2D
    ax = plot_2D(
        transpose(speeds),
        transpose(Ydatas),
        legend_list=legends,
        color_list=colors[0 : Norders + 1],
        linestyle_list=linestyles[0 : Norders + 1],
        save_path=save_path,
        y_min=y_min,
        y_max=y_max,
        xlabel=xlabel,
        ylabel=ylabel,
        title=title,
        is_disp_title=is_disp_title,
        is_show_fig=is_show_fig,
        fig=fig,
        ax=ax,
    )

    return ax



import matplotlib.pyplot as plt
from numpy import vstack, linspace, max as np_max

from ...Functions.init_fig import init_subplot, init_fig
from ...definitions import config_dict

COLORMAP = config_dict["PLOT"]["COLOR_DICT"]["COLOR_MAP"]
FONT_NAME = config_dict["PLOT"]["FONT_NAME"]
FONT_SIZE_TITLE = config_dict["PLOT"]["FONT_SIZE_TITLE"]
FONT_SIZE_LABEL = config_dict["PLOT"]["FONT_SIZE_LABEL"]
FONT_SIZE_LEGEND = config_dict["PLOT"]["FONT_SIZE_LEGEND"]


def plot_spectrogram(
    speeds,
    freqs,
    A_orders,
    freq_max=10000,
    order_width=15,
    db_min=0,
    db_max=None,
    save_path=None,
    Norders=100,
    zlabel=None,
    title=None,
    is_disp_title=True,
    type_spectro=0,
    is_reverse_spectro=False,
    speed_max=None,
    is_show_fig=None,
    fig=None,
    ax=None,
    is_db=True,
):
    """
    Plot spectrogram from a variable speed simulation

    speeds: ndarray
        Array of speed values
    freqs : ndarray
        Array of frequency vectors for each speed
    A_orders : ndarray
        Amplitude array per speeds and orders(Nspeeds, Norders)
    freq_max : float
        maximum frequency
    order_width : float
        width of the order lines
    db_min : float
        lines under db_min are not displayed
    db_max : float
        maximum dB scale
    save_path : str
        location to store the figure
    Norders : integer
        number of orders
    xlabel : str
        label for the x-axis
    ylabel : str
        label for the y-axis
    zlabel : str
        label for the z-axis
    title : str
        title of the graph
    is_disp_title : bool
        boolean indicating if the title must be displayed
    type_spectro : int
        type of spectrogram (0: per order, 1: per speed)
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        ax on which to plot the data

    Returns
    -------

    Raises
    ------
    PlotError
    """

    # Create figure if needed
    if fig is None and ax is None:
        fig, ax = init_subplot(fig=None)

    colorbar = None

    # Number of orders to plot
    Norders = min([Norders, freqs.shape[1]])

    if is_db:
        if db_max is None:
            db_max = int(1.05 * np_max(A_orders.flatten("C")))

    if is_reverse_spectro:
        xlabel = "Rotation speed [rpm]"
        ylabel = "Frequency [Hz]"
    else:
        xlabel = "Frequency [Hz]"
        ylabel = "Rotation speed [rpm]"

    if type_spectro == 0:  # plot order per order
        # Duplicate speed vector to plot a pcolormesh
        if is_reverse_spectro:
            xx = vstack((speeds, speeds))
        else:
            yy = vstack((speeds, speeds))

        # Loop on orders
        for k in range(Norders):
            if is_reverse_spectro:
                y = freqs[:, k]
                yy = vstack((y - order_width, y + order_width))
            else:
                x = freqs[:, k]
                xx = vstack((x - order_width, x + order_width))
            c = A_orders[:, k]
            cc = vstack((c, c))
            if np_max(c) > db_min:
                colorbar = ax.pcolormesh(
                    xx,
                    yy,
                    cc,
                    cmap=COLORMAP,
                    vmin=db_min,
                    vmax=db_max,
                    shading="gouraud",
                )
    elif type_spectro == 1:  # plot speed by speed  #TODO

        print("type_spectro==1 not available yet")

        pass
        # for i in range(len(freqs)):
        #     c = ax.scatter(
        #         freqs[i],
        #         speeds,
        #         c=A_orders[i],
        #         vmin=db_min,
        #         vmax=db_max,
        #         marker="s",
        #         cmap=COLORMAP,
        #     )

    if is_db:
        v = linspace(db_min, db_max, int((db_max - db_min) / 10) + 1, endpoint=True)
        if colorbar is not None:
            clb = fig.colorbar(colorbar, ticks=v, format="%.4g")
    else:
        if colorbar is not None:
            clb = fig.colorbar(colorbar, format="%.4g")
    if colorbar is not None:
        clb.ax.set_title(zlabel, fontsize=FONT_SIZE_LEGEND, fontname=FONT_NAME)
        clb.ax.tick_params(labelsize=FONT_SIZE_LEGEND)

        for l in clb.ax.yaxis.get_ticklabels():
            l.set_family(FONT_NAME)
    if is_reverse_spectro:
        ax.set_ylim([0, freq_max])
        if speed_max is not None:
            ax.set_xlim([0, speed_max])
    else:
        ax.set_xlim([0, freq_max])
        if speed_max is not None:
            ax.set_ylim([0, speed_max])

    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)

    if is_disp_title:
        ax.set_title(title)

    for item in (
        [ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()
    ):
        item.set_fontname(FONT_NAME)
        item.set_fontsize(FONT_SIZE_LABEL)
    ax.title.set_fontname(FONT_NAME)
    ax.title.set_fontsize(FONT_SIZE_TITLE)

    if save_path is not None:
        fig.savefig(save_path)
        plt.close()

    if is_show_fig:
        fig.show()



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/DataLinspace.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/SciDataTool/tree/master/SciDataTool/Methods//DataLinspace
"""

from os import linesep
from sys import getsizeof
from ._check import check_var, raise_
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Data import Data

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.DataLinspace.get_values import get_values
except ImportError as error:
    get_values = error

try:
    from ..Methods.DataLinspace.get_length import get_length
except ImportError as error:
    get_length = error

try:
    from ..Methods.DataLinspace.get_axis_periodic import get_axis_periodic
except ImportError as error:
    get_axis_periodic = error

try:
    from ..Methods.DataLinspace.has_period import has_period
except ImportError as error:
    has_period = error

try:
    from ..Methods.DataLinspace.get_periodicity import get_periodicity
except ImportError as error:
    get_periodicity = error


from ._check import InitUnKnowClassError


class DataLinspace(Data):
    """Class for axes defined as linspaces"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.DataLinspace.get_values
    if isinstance(get_values, ImportError):
        get_values = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataLinspace method get_values: " + str(get_values)
                )
            )
        )
    else:
        get_values = get_values
    # cf Methods.DataLinspace.get_length
    if isinstance(get_length, ImportError):
        get_length = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataLinspace method get_length: " + str(get_length)
                )
            )
        )
    else:
        get_length = get_length
    # cf Methods.DataLinspace.get_axis_periodic
    if isinstance(get_axis_periodic, ImportError):
        get_axis_periodic = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataLinspace method get_axis_periodic: "
                    + str(get_axis_periodic)
                )
            )
        )
    else:
        get_axis_periodic = get_axis_periodic
    # cf Methods.DataLinspace.has_period
    if isinstance(has_period, ImportError):
        has_period = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataLinspace method has_period: " + str(has_period)
                )
            )
        )
    else:
        has_period = has_period
    # cf Methods.DataLinspace.get_periodicity
    if isinstance(get_periodicity, ImportError):
        get_periodicity = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use DataLinspace method get_periodicity: "
                    + str(get_periodicity)
                )
            )
        )
    else:
        get_periodicity = get_periodicity
    # save and copy methods are available in all object
    save = save
    copy = copy

    def __init__(
        self,
        initial=None,
        final=None,
        step=None,
        number=None,
        include_endpoint=True,
        is_components=False,
        symmetries=-1,
        symbol="",
        name="",
        unit="",
        normalizations=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for SciDataTool type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for SciDataTool Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "initial" in list(init_dict.keys()):
                initial = init_dict["initial"]
            if "final" in list(init_dict.keys()):
                final = init_dict["final"]
            if "step" in list(init_dict.keys()):
                step = init_dict["step"]
            if "number" in list(init_dict.keys()):
                number = init_dict["number"]
            if "include_endpoint" in list(init_dict.keys()):
                include_endpoint = init_dict["include_endpoint"]
            if "is_components" in list(init_dict.keys()):
                is_components = init_dict["is_components"]
            if "symmetries" in list(init_dict.keys()):
                symmetries = init_dict["symmetries"]
            if "symbol" in list(init_dict.keys()):
                symbol = init_dict["symbol"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "unit" in list(init_dict.keys()):
                unit = init_dict["unit"]
            if "normalizations" in list(init_dict.keys()):
                normalizations = init_dict["normalizations"]
        # Set the properties (value check and convertion are done in setter)
        self.initial = initial
        self.final = final
        self.step = step
        self.number = number
        self.include_endpoint = include_endpoint
        self.is_components = is_components
        self.symmetries = symmetries
        # Call Data init
        super(DataLinspace, self).__init__(
            symbol=symbol, name=name, unit=unit, normalizations=normalizations
        )
        # The class is frozen (in Data init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        DataLinspace_str = ""
        # Get the properties inherited from Data
        DataLinspace_str += super(DataLinspace, self).__str__()
        DataLinspace_str += "initial = " + str(self.initial) + linesep
        DataLinspace_str += "final = " + str(self.final) + linesep
        DataLinspace_str += "step = " + str(self.step) + linesep
        DataLinspace_str += "number = " + str(self.number) + linesep
        DataLinspace_str += "include_endpoint = " + str(self.include_endpoint) + linesep
        DataLinspace_str += "is_components = " + str(self.is_components) + linesep
        DataLinspace_str += "symmetries = " + str(self.symmetries) + linesep
        return DataLinspace_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Data
        if not super(DataLinspace, self).__eq__(other):
            return False
        if other.initial != self.initial:
            return False
        if other.final != self.final:
            return False
        if other.step != self.step:
            return False
        if other.number != self.number:
            return False
        if other.include_endpoint != self.include_endpoint:
            return False
        if other.is_components != self.is_components:
            return False
        if other.symmetries != self.symmetries:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Data
        diff_list.extend(super(DataLinspace, self).compare(other, name=name))
        if other._initial != self._initial:
            diff_list.append(name + ".initial")
        if other._final != self._final:
            diff_list.append(name + ".final")
        if other._step != self._step:
            diff_list.append(name + ".step")
        if other._number != self._number:
            diff_list.append(name + ".number")
        if other._include_endpoint != self._include_endpoint:
            diff_list.append(name + ".include_endpoint")
        if other._is_components != self._is_components:
            diff_list.append(name + ".is_components")
        if other._symmetries != self._symmetries:
            diff_list.append(name + ".symmetries")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Data
        S += super(DataLinspace, self).__sizeof__()
        S += getsizeof(self.initial)
        S += getsizeof(self.final)
        S += getsizeof(self.step)
        S += getsizeof(self.number)
        S += getsizeof(self.include_endpoint)
        S += getsizeof(self.is_components)
        if self.symmetries is not None:
            for key, value in self.symmetries.items():
                S += getsizeof(value) + getsizeof(key)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Data
        DataLinspace_dict = super(DataLinspace, self).as_dict(**kwargs)
        DataLinspace_dict["initial"] = self.initial
        DataLinspace_dict["final"] = self.final
        DataLinspace_dict["step"] = self.step
        DataLinspace_dict["number"] = self.number
        DataLinspace_dict["include_endpoint"] = self.include_endpoint
        DataLinspace_dict["is_components"] = self.is_components
        DataLinspace_dict["symmetries"] = (
            self.symmetries.copy() if self.symmetries is not None else None
        )
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        DataLinspace_dict["__class__"] = "DataLinspace"
        return DataLinspace_dict

    def _set_None(self):
        """Set all the properties to None (except SciDataTool object)"""

        self.initial = None
        self.final = None
        self.step = None
        self.number = None
        self.include_endpoint = None
        self.is_components = None
        self.symmetries = None
        # Set to None the properties inherited from Data
        super(DataLinspace, self)._set_None()

    def _get_initial(self):
        """getter of initial"""
        return self._initial

    def _set_initial(self, value):
        """setter of initial"""
        check_var("initial", value, "float")
        self._initial = value

    initial = property(
        fget=_get_initial,
        fset=_set_initial,
        doc=u"""First value

        :Type: float
        """,
    )

    def _get_final(self):
        """getter of final"""
        return self._final

    def _set_final(self, value):
        """setter of final"""
        check_var("final", value, "float")
        self._final = value

    final = property(
        fget=_get_final,
        fset=_set_final,
        doc=u"""Last value

        :Type: float
        """,
    )

    def _get_step(self):
        """getter of step"""
        return self._step

    def _set_step(self, value):
        """setter of step"""
        check_var("step", value, "float")
        self._step = value

    step = property(
        fget=_get_step,
        fset=_set_step,
        doc=u"""Step

        :Type: float
        """,
    )

    def _get_number(self):
        """getter of number"""
        return self._number

    def _set_number(self, value):
        """setter of number"""
        check_var("number", value, "int")
        self._number = value

    number = property(
        fget=_get_number,
        fset=_set_number,
        doc=u"""Number of steps

        :Type: int
        """,
    )

    def _get_include_endpoint(self):
        """getter of include_endpoint"""
        return self._include_endpoint

    def _set_include_endpoint(self, value):
        """setter of include_endpoint"""
        check_var("include_endpoint", value, "bool")
        self._include_endpoint = value

    include_endpoint = property(
        fget=_get_include_endpoint,
        fset=_set_include_endpoint,
        doc=u"""Boolean indicating if the endpoint must be included

        :Type: bool
        """,
    )

    def _get_is_components(self):
        """getter of is_components"""
        return self._is_components

    def _set_is_components(self, value):
        """setter of is_components"""
        check_var("is_components", value, "bool")
        self._is_components = value

    is_components = property(
        fget=_get_is_components,
        fset=_set_is_components,
        doc=u"""Boolean inidcating if the axis is components

        :Type: bool
        """,
    )

    def _get_symmetries(self):
        """getter of symmetries"""
        return self._symmetries

    def _set_symmetries(self, value):
        """setter of symmetries"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("symmetries", value, "dict")
        self._symmetries = value

    symmetries = property(
        fget=_get_symmetries,
        fset=_set_symmetries,
        doc=u"""Dictionary of the symmetries along each axis, used to reduce storage

        :Type: dict
        """,
    )



from numpy import zeros, array, arange, meshgrid, squeeze, abs as np_abs, max as np_max

from ...Functions.Plot.plot_4D import plot_4D


def plot_modal_proj(
    modal_proj,
    mode_labels,
    nat_freqs,
    save_path=None,
    is_show_fig=True,
    label="",
):
    """Plot the result of the modal projection at a given frequency

    Parameters
    ----------
    modal_cont : dict
        result of the modal contribution
    mode_labels : list
        list of mode labels
    nat_freqs : list
        list of natural frequencies
    save_path : str
        location to store the figure
    is_show_fig : bool
        True to call show at the end of the plot
    label : str
        name of the FRF
    """

    title = "Modal projection of " + label

    nloads = len(modal_proj)
    nmodes = len(mode_labels)
    Zdatas = zeros((nloads, nmodes))
    annot = zeros((nloads, nmodes), dtype=int)
    yticklabels = []
    size = zeros((nloads, nmodes))
    i = 0
    for key in modal_proj.keys():
        Zdatas[i, :] = squeeze(np_abs(array(modal_proj[key])))
        yticklabels.append(key)
        size[i, :] = 1000 * Zdatas[i, :] / np_max(Zdatas[i, :])
        annot[i, :] = (array(nat_freqs)).astype(int)
        i += 1
    x = arange(nmodes)
    y = arange(nloads)
    x_map, y_map = meshgrid(x, y)
    x_flat = x_map.flatten()
    y_flat = y_map.flatten()
    Zdata_flat = Zdatas.flatten()
    size_flat = size.flatten()
    zmax = max(Zdata_flat)
    annotations = annot.flatten()

    xticklabels = mode_labels

    plot_4D(
        x_flat,
        y_flat,
        Zdata_flat,
        size_flat,
        z_max=zmax,
        z_min=0,
        xticks=x,
        yticks=y,
        xticklabels=xticklabels,
        yticklabels=yticklabels,
        xlabel="Modes",
        ylabel="Loadcases",
        zlabel=label,
        annotations=annotations,
        type="scatter",
        title=title,
        save_path=save_path,
        is_show_fig=is_show_fig,
    )
